@@ -10,6 +10,7 @@ from typing import (
     Awaitable,
     AsyncIterable,
     Callable,
+    Coroutine,
     Any,
     overload,
     Optional,
@@ -34,7 +35,7 @@ class _BorrowedAsyncIterator(AsyncGenerator[T, S]):
     __slots__ = "__wrapped__", "__anext__", "asend", "athrow", "_wrapper"
 
     # Type checker does not understand `__slot__` definitions
-    __anext__: Callable[[Any], Awaitable[T]]
+    __anext__: Callable[[Any], Coroutine[Any, Any, T]]
     asend: Any
     athrow: Any
 
@@ -76,7 +77,7 @@ class _BorrowedAsyncIterator(AsyncGenerator[T, S]):
         if hasattr(self, "athrow"):
             self.athrow = wrapper_iterator.athrow
 
-    def aclose(self) -> Awaitable[None]:
+    def aclose(self) -> Coroutine[Any, Any, None]:
         return self._aclose_wrapper()
 
 
@@ -118,7 +119,17 @@ class _ScopedAsyncIteratorContext(AsyncContextManager[AsyncIterator[T]]):
         return f"<{self.__class__.__name__} of {self._iterator!r} at 0x{(id(self)):x}>"
 
 
-def borrow(iterator: AsyncIterator[T], /) -> AsyncIterator[T]:
+@overload
+def borrow(iterator: AsyncGenerator[T, S], /) -> AsyncGenerator[T, S]: ...
+
+
+@overload
+def borrow(iterator: AsyncIterator[T], /) -> AsyncIterator[T]: ...
+
+
+def borrow(
+    iterator: Union[AsyncIterator[T], AsyncGenerator[T, Any]], /
+) -> Union[AsyncIterator[T], AsyncGenerator[T, Any]]:
     """
     Borrow an async iterator, preventing to ``aclose`` it
 
@@ -141,7 +152,7 @@ def borrow(iterator: AsyncIterator[T], /) -> AsyncIterator[T]:
             "borrowing requires an async iterator "
             + f"with __aiter__ and __anext__ method, got {type(iterator).__name__}"
         )
-    return _BorrowedAsyncIterator(iterator)
+    return _BorrowedAsyncIterator[T, Any](iterator)
 
 
 def scoped_iter(iterable: AnyIterable[T], /) -> AsyncContextManager[AsyncIterator[T]]:
