@@ -32,8 +32,29 @@ async def test_contextmanager():
         assert context == 1337
 
 
+@sync
+async def test_contextmanager_decorator():
+    witness: list[str] = []
+
+    @a.contextmanager
+    async def decorator():
+        witness.append("enter")
+        yield
+        witness.append("exit")
+
+    @decorator()
+    async def func():
+        witness.append("inner")
+
+    for repetition in range(1, 4):
+        await func()
+        assert witness == ["enter", "inner", "exit"] * repetition
+
+
 @sync
 async def test_contextmanager_no_yield():
+    """Test that it is an error for a context to not yield"""
+
     @a.contextmanager
     async def no_yield():
         if False:
@@ -46,6 +67,8 @@ async def test_contextmanager_no_yield():
 
 @sync
 async def test_contextmanager_no_stop():
+    """Test that it is an error for a context to yield again after stopping"""
+
     @a.contextmanager
     async def no_stop():
         yield
@@ -59,7 +82,7 @@ async def test_contextmanager_no_stop():
     async def supress_no_stop():
         try:
             yield
-        except BaseException:
+        except BaseException:  # noqa: B036
             yield
 
     with pytest.raises(RuntimeError):
@@ -69,6 +92,8 @@ async def test_contextmanager_no_stop():
 
 @sync
 async def test_contextmanager_raise_asyncstop():
+    """Test that StopAsyncIteration may propagate out of a context block"""
+
     @a.contextmanager
     async def no_raise():
         yield
@@ -113,6 +138,8 @@ async def test_contextmanager_raise_runtimeerror():
 
 @sync
 async def test_contextmanager_raise_same():
+    """Test that outer exceptions do not shadow inner/newer ones"""
+
     @a.contextmanager
     async def reraise():
         try:
@@ -128,7 +155,7 @@ async def test_contextmanager_raise_same():
     async def recreate():
         try:
             yield
-        except BaseException as err:
+        except BaseException as err:  # noqa: B036
             raise type(err)("inside") from None
 
     with pytest.raises(KeyError, match="inside"):
@@ -136,6 +163,66 @@ async def test_contextmanager_raise_same():
             raise KeyError("outside")
 
 
+@sync
+async def test_contextmanager_raise_generatorexit():
+    """Test that shutdown via GeneratorExit is propagated"""
+
+    @a.contextmanager
+    async def no_op():
+        yield
+
+    with pytest.raises(GeneratorExit):
+        async with no_op():
+            raise GeneratorExit("used to tear down coroutines")
+
+    # during shutdown, generators may be killed in arbitrary order
+    # make sure we do not suppress GeneratorExit
+    context = no_op()
+    with pytest.raises(GeneratorExit, match="inner"):
+        async with context:
+            # simulate cleanup closing the child early
+            await context.gen.aclose()
+            raise GeneratorExit("inner")
+
+
+@sync
+async def test_contextmanager_no_suppress_generatorexit():
+    """Test that GeneratorExit is not suppressed"""
+
+    @a.contextmanager
+    async def no_op():
+        yield
+
+    exc = GeneratorExit("GE should not be replaced normally")
+    with pytest.raises(type(exc)) as exc_info:
+        async with no_op():
+            raise exc
+    assert exc_info.value is exc
+
+    @a.contextmanager
+    async def exit_ge():
+        try:
+            yield
+        except GeneratorExit:
+            # treat GE as regular exit
+            pass
+
+    with pytest.raises(GeneratorExit):
+        async with exit_ge():
+            raise GeneratorExit("Resume teardown if child exited")
+
+    @a.contextmanager
+    async def ignore_ge():
+        try:
+            yield
+        except GeneratorExit:
+            yield
+
+    with pytest.raises(RuntimeError):
+        async with ignore_ge():
+            raise GeneratorExit("Warn if child does not exit")
+
+
 @sync
 async def test_nullcontext():
     async with a.nullcontext(1337) as value:
@@ -226,7 +313,7 @@ async def test_exit_stack_push():
     async def suppress():
         try:
             yield
-        except BaseException as exc_val:
+        except BaseException as exc_val:  # noqa: B036
             seen.append(exc_val)
 
     async def replace(exc_type, exc_val, tb, new):
